
import threading, os, time




def init():
    t = threading.Thread(target=printParam)
    t.setDaemon(True)
    t.start()



def sendParam():




def main():
    os.environ["WHATAP_HOME"] = "./"
    net.startNet()


